package com.modcrafting.security;

import java.security.GeneralSecurityException;
import java.security.MessageDigest;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.logging.Level;

import org.bukkit.configuration.file.YamlConfiguration;

public class SQLDatabases{
	static SecurityDragon plugin;
	
	public Connection getSQLConnection() {
		YamlConfiguration Config = (YamlConfiguration) plugin.getConfig();
		String mysqlDatabase = Config.getString("mysql-database","jdbc:mysql://localhost:3306/minecraft");
		String mysqlUser = Config.getString("mysql-user","root");
		String mysqlPassword = Config.getString("mysql-password","root");
			try {

				return DriverManager.getConnection(mysqlDatabase + "?autoReconnect=true&user=" + mysqlUser + "&password=" + mysqlPassword);
			} catch (SQLException ex) {
			
				SecurityDragon.log.log(Level.SEVERE, "Unable to retreive connection", ex);
			}
			return null;
	}

	
	public void initialize(SecurityDragon plugin){
		YamlConfiguration Config = (YamlConfiguration) plugin.getConfig();
		String mysqlTable = Config.getString("mysql-table");
		SQLDatabases.plugin = plugin;
		Connection conn = getSQLConnection();
		
		
		if (conn == null) {
			SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Could not establish SQL connection. Disabling SecurityDragon");
			SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Adjust Settings in Config or set MySql: False");
			plugin.getServer().getPluginManager().disablePlugin(plugin);
			return;
		} else {
			PreparedStatement ps = null;
			ResultSet rs = null;
			
			try {
				ps = conn.prepareStatement("SELECT * FROM " + mysqlTable);
				try{
					
					DatabaseMetaData dbm = conn.getMetaData();
					rs = dbm.getTables(null, null, mysqlTable, null);
		            	if (!rs.next()){
		            		conn.setAutoCommit(false);
		            		conn.commit();
		            		SecurityDragon.log.log(Level.INFO, "[SecurityDragon]: Table " + mysqlTable + " created.");
		            	}
		            	rs = ps.executeQuery();
							            
				} catch (SQLException ex) {
					SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Database Error: No Table Found");
                }
				
				try {
					while (rs.next()){
					String pName = rs.getString("name").toLowerCase();
					plugin.allowedPlayers.add(pName);
					}
				}catch (NullPointerException ex){
					SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Detected Major issues with database.");
					plugin.getServer().getPluginManager().disablePlugin(plugin);
					SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Attempting Restart.");
					plugin.getServer().getPluginManager().enablePlugin(plugin);
					return;
				}
			} catch (SQLException ex) {
				SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Couldn't execute MySQL statement: ", ex);
				return;
			} finally {
				try {
					if (ps != null)
						ps.close();
					if (rs != null)
						rs.close();
					if (conn != null)
						conn.close();
				} catch (SQLException ex) {
					SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Failed to close MySQL connection: ", ex);
				}
			}	

			try {
				if (!plugin.isEnabled()){
					return;
				}
				conn.close();
				SecurityDragon.log.log(Level.INFO, "[SecurityDragon] Initialized db connection" );
			} catch (SQLException e) {
				e.printStackTrace();
				plugin.getServer().getPluginManager().disablePlugin(plugin);
			}
		}
		
	}
	public boolean removeFromlist(String player) {
		YamlConfiguration Config = (YamlConfiguration) plugin.getConfig();

		String mysqlTable = Config.getString("mysql-table");

		Connection conn = null;
		PreparedStatement ps = null;
		try {
			conn = getSQLConnection();
			ps = conn.prepareStatement("DELETE FROM " + mysqlTable + " WHERE username = ? ORDER BY username DESC LIMIT 1");
			ps.setString(1, player);
			ps.executeUpdate();
		} catch (SQLException ex) {
			SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Couldn't execute MySQL statement: ", ex);
			return false;
		} finally {
			try {
				if (ps != null)
					ps.close();
				if (conn != null)
					conn.close();
			} catch (SQLException ex) {
				SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Failed to close MySQL connection: ", ex);
			}
		}
		return true;

	}
	public static boolean check(String passwd,String dbEntry) {
	      if (passwd==null || dbEntry==null || dbEntry.length()==0)
	    throw new IllegalArgumentException();
	      String[] arr = dbEntry.split(":",2);
	      if (arr.length==2) {
	    // new format as {HASH}:{SALT}
	    String cryptpass = arr[0];
	    String salt = arr[1];
	   
	    return md5(passwd+salt).equals(cryptpass);
	      } else {
	         // old format as {HASH} just like PHPbb and many other apps
	    String cryptpass = dbEntry;
	     
	    return md5(passwd).equals(cryptpass);
	      }
	   }
	private static String md5(String data) {
	      byte[] bdata = new byte[data.length()]; int i; byte[] hash;

	      for (i=0;i<data.length();i++) bdata[i]=(byte)(data.charAt(i)&0xff );

	      try {
	         MessageDigest md5er = MessageDigest.getInstance("MD5");
	         hash = md5er.digest(bdata);
	      } catch (GeneralSecurityException e) { throw new RuntimeException(e); }

	      StringBuffer r = new StringBuffer(32);
	      for (i=0;i<hash.length;i++) {
	         String x = Integer.toHexString(hash[i]&0xff);
	         if (x.length()<2) r.append("0");
	         r.append(x);
	      }
	      return r.toString();     
	   }
	public String getName(String sender) {
		YamlConfiguration Config = (YamlConfiguration) plugin.getConfig();
		Connection conn = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		String logip = Config.getString("mysql-table");
		try {
			conn = getSQLConnection();
			ps = conn.prepareStatement("SELECT * FROM " + logip + " WHERE username = ?");
			ps.setString(1, sender);
			rs = ps.executeQuery();
			while (rs.next()){
				String name = rs.getString("name");
				return name;
			}
		} catch (SQLException ex) {
			SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Couldn't find player.");
		} finally {
			try {
				if (ps != null)
					ps.close();
				if (conn != null)
					conn.close();
				if (rs != null)
					rs.close();
			} catch (SQLException ex) {
				SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Failed to close MySQL connection: ", ex);
			}
		}
		return null;
	}
	public String login(String sender, String pass) {
		YamlConfiguration Config = (YamlConfiguration) plugin.getConfig();
		Connection conn = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		String logip = Config.getString("mysql-table");
		try {
			conn = getSQLConnection();
			ps = conn.prepareStatement("SELECT * FROM " + logip + " WHERE username = ?");
			ps.setString(1, sender);
			rs = ps.executeQuery();
			while (rs.next()){
				String pswd = rs.getString("password");
				String name = rs.getString("username");
				if (check(pass,pswd)){
					return name;
				}else{
				return null;
				}
			}
		} catch (SQLException ex) {
			SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Couldn't find player.");
		} finally {
			try {
				if (ps != null)
					ps.close();
				if (conn != null)
					conn.close();
				if (rs != null)
					rs.close();
			} catch (SQLException ex) {
				SecurityDragon.log.log(Level.SEVERE, "[SecurityDragon] Failed to close MySQL connection: ", ex);
			}
		}
		return null;
	}
	
}
